@@ -18,6 +18,6 @@
 #' demo("mandelbrot", package = "future", ask = FALSE)
 #' }
 #'
-#' @docType package
 #' @name future.batchtools
-NULL
+#' @aliases future.batchtools-package
+"_PACKAGE"
